<?php

namespace Btravel\Cards\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Btravel\Cards\Http\Requests\CardOffersRequest;
use Btravel\Cards\Interfaces\CardOffersRepositoryInterface;
use Btravel\Cards\Models\CardOffers;

/**
 * Resource controller class for card_offers.
 */
class CardOffersResourceController extends BaseController
{

    /**
     * Initialize card_offers resource controller.
     *
     * @param type CardOffersRepositoryInterface $card_offers
     *
     * @return null
     */
    public function __construct(CardOffersRepositoryInterface $card_offers)
    {
        parent::__construct();
        $this->repository = $card_offers;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Btravel\Cards\Repositories\Criteria\CardOffersResourceCriteria::class);
    }

    /**
     * Display a list of card_offers.
     *
     * @return Response
     */
    public function index(CardOffersRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Btravel\Cards\Repositories\Presenter\CardOffersPresenter::class)
                ->$function();
        }

        $card_offers = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('cards::card_offers.names'))
            ->view('cards::card_offers.index', true)
            ->data(compact('card_offers', 'view'))
            ->output();
    }

    /**
     * Display card_offers.
     *
     * @param Request $request
     * @param Model   $card_offers
     *
     * @return Response
     */
    public function show(CardOffersRequest $request, CardOffers $card_offers)
    {

        if ($card_offers->exists) {
            $view = 'cards::card_offers.show';
        } else {
            $view = 'cards::card_offers.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('cards::card_offers.name'))
            ->data(compact('card_offers'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new card_offers.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CardOffersRequest $request)
    {

        $card_offers = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('cards::card_offers.name')) 
            ->view('cards::card_offers.create', true) 
            ->data(compact('card_offers'))
            ->output();
    }

    /**
     * Create new card_offers.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CardOffersRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $card_offers                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('cards::card_offers.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('cards/card_offers/' . $card_offers->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/cards/card_offers'))
                ->redirect();
        }

    }

    /**
     * Show card_offers for editing.
     *
     * @param Request $request
     * @param Model   $card_offers
     *
     * @return Response
     */
    public function edit(CardOffersRequest $request, CardOffers $card_offers)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('cards::card_offers.name'))
            ->view('cards::card_offers.edit', true)
            ->data(compact('card_offers'))
            ->output();
    }

    /**
     * Update the card_offers.
     *
     * @param Request $request
     * @param Model   $card_offers
     *
     * @return Response
     */
    public function update(CardOffersRequest $request, CardOffers $card_offers)
    {
        try {
            $attributes = $request->all();

            $card_offers->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('cards::card_offers.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('cards/card_offers/' . $card_offers->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('cards/card_offers/' . $card_offers->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the card_offers.
     *
     * @param Model   $card_offers
     *
     * @return Response
     */
    public function destroy(CardOffersRequest $request, CardOffers $card_offers)
    {
        try {

            $card_offers->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('cards::card_offers.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('cards/card_offers/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('cards/card_offers/' . $card_offers->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple card_offers.
     *
     * @param Model   $card_offers
     *
     * @return Response
     */
    public function delete(CardOffersRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('cards::card_offers.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('cards/card_offers'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/cards/card_offers'))
                ->redirect();
        }

    }

    /**
     * Restore deleted card_offers.
     *
     * @param Model   $card_offers
     *
     * @return Response
     */
    public function restore(CardOffersRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('cards::card_offers.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/cards/card_offers'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/cards/card_offers/'))
                ->redirect();
        }

    }

}
